﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace howto_round_robin_home_teams
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private const int BYE = -1;

        // Schedule the round robin.
        private void btnGo_Click(object sender, EventArgs e)
        {
            // Get the teams.
            string all_teams = txtTeams.Text;
            char[] separators = { '\r', '\n' };
            string[] team_names = all_teams.Split(separators,
                StringSplitOptions.RemoveEmptyEntries);

            // Get the schedule.
            int num_teams = team_names.Length;
            bool[,] is_home;
            int[,] results = GenerateRoundRobin(num_teams, out is_home);

            // Keep track of number of home games.
            int[] num_home = new int[num_teams];

            // Display the result.
            string txt = "";
            for (int round = 0; round <= results.GetUpperBound(1); round++)
            {
                txt += "Round " + round + ":\r\n";
                for (int team1 = 0; team1 < num_teams; team1++)
                {
                    int team2 = results[team1, round];
                    if (team2 == BYE)
                    {
                        txt += "    " + team_names[team1] + " (bye)\r\n";
                    }
                    else if (team1 < team2)
                    {
                        if (is_home[team1, round])
                        {
                            txt += "    " +
                                "[" + team_names[team1] + "] v " +
                                team_names[team2] + "\r\n";
                            num_home[team1]++;
                        }
                        else
                        {
                            txt += "    " +
                                team_names[team1] + " v " +
                                "[" + team_names[team2] + "]\r\n";
                            num_home[team2]++;
                        }
                    }
                }
            }

            // Verify the home game counts.
            Console.WriteLine("**********");
            for (int team = 0; team < num_teams; team++)
            {
                Console.WriteLine(team_names[team] + " is home " + num_home[team] + " times.");
            }

            txtResults.Text = txt;
        }

        // Return an array where results(i, j) gives
        // the opponent of team i in round j.
        private int[,] GenerateRoundRobin(int num_teams, out bool[,] is_home)
        {
            if (num_teams % 2 == 0)
                return GenerateRoundRobinEven(num_teams, out is_home);
            else
                return GenerateRoundRobinOdd(num_teams, out is_home);
        }

        // Return an array where results[i, j] gives
        // the opponent of team i in round j.
        // The value is_home[i, j] is true if
        // team i is the home team in round j.
        // Note: num_teams must be odd.
        private int[,] GenerateRoundRobinOdd(int num_teams, out bool[,] is_home)
        {
            int n2 = (int)((num_teams - 1) / 2);
            int[,] results = new int[num_teams, num_teams];
            is_home = new bool[num_teams, num_teams];

            // Initialize the list of teams.
            int[] teams = new int[num_teams];
            for (int i = 0; i < num_teams; i++) teams[i] = i;

            // Start the rounds.
            for (int round = 0; round < num_teams; round++)
            {
                for (int i = 0; i < n2; i++)
                {
                    int team1 = teams[n2 - i];
                    int team2 = teams[n2 + i + 1];
                    results[team1, round] = team2;
                    results[team2, round] = team1;

                    // See which is the home team.
                    is_home[team1, round] = (i % 2 == 0);
                    is_home[team2, round] = !is_home[team1, round];
                }

                // Set the team with the bye.
                results[teams[0], round] = BYE;
                is_home[teams[0], round] = true;

                // Rotate the array.
                RotateArray(teams);
            }

            return results;
        }

        // Return an array where results(i, j) gives
        // the opponent of team i in round j.
        // Note: num_teams must be even.
        private int[,] GenerateRoundRobinEven(int num_teams, out bool[,] is_home)
        {
            // Generate the result for one fewer teams.
            bool[,] odd_is_home;
            int[,] odd_results = GenerateRoundRobinOdd(num_teams - 1, out odd_is_home);

            // Copy the results into bigger arrays,
            // replacing the byes with the extra team.
            int[,] results = new int[num_teams, num_teams - 1];
            is_home = new bool[num_teams, num_teams - 1];
            for (int team = 0; team < num_teams - 1; team++)
            {
                for (int round = 0; round < num_teams - 1; round++)
                {
                    if (odd_results[team, round] == BYE)
                    {
                        // Change the bye to the new team.
                        results[team, round] = num_teams - 1;
                        results[num_teams - 1, round] = team;

                        // Give every other team an extra home game.
                        is_home[team, round] = (team % 2 == 0);
                        is_home[num_teams - 1, round] = !is_home[team, round];
                    }
                    else
                    {
                        results[team, round] = odd_results[team, round];
                        is_home[team, round] = odd_is_home[team, round];
                    }
                }
            }

            return results;
        }

        // Rotate the entries one position.
        private void RotateArray(int[] teams)
        {
            int tmp = teams[teams.Length - 1];
            Array.Copy(teams, 0, teams, 1, teams.Length - 1);
            teams[0] = tmp;
        }
    }
}
